==> player-animation-lib-2.0.3%2B1.21.1-sources.jar: dev/kosmx/playerAnim/mixin/firstPerson/LevelRendererMixin.java <==
package dev.kosmx.playerAnim.mixin.firstPerson;

import dev.kosmx.playerAnim.api.firstPerson.FirstPersonMode;
import dev.kosmx.playerAnim.impl.IAnimatedPlayer;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_310;
import net.minecraft.class_4184;
import net.minecraft.class_4587;
import net.minecraft.class_4597;
import net.minecraft.class_757;
import net.minecraft.class_761;
import net.minecraft.class_765;
import net.minecraft.class_9779;
import org.joml.Matrix4f;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_761.class)
public class LevelRendererMixin {

    @Unique
    private boolean defaultCameraState = false;

    // @Redirect(at = @At(target = "Lnet/minecraft/client/Camera;isDetached()Z")) is forbidden

    @Inject(method = "renderLevel", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/Camera;isDetached()Z"))
    private void fakeThirdPersonMode(class_9779 deltaTracker, boolean bl, class_4184 camera, class_757 gameRenderer, class_765 lightTexture, Matrix4f matrix4f, Matrix4f matrix4f2, CallbackInfo ci) {
        // mods may need to redirect that method, I want to avoid compatibility issues as long as possible
        defaultCameraState = camera.method_19333();
        if (camera.method_19331() instanceof IAnimatedPlayer player && (player.playerAnimator_getAnimation().getFirstPersonMode() == FirstPersonMode.THIRD_PERSON_MODEL)) {
            FirstPersonMode.setFirstPersonPass(!camera.method_19333() && (!(camera.method_19331() instanceof class_1309) || !((class_1309) camera.method_19331()).method_6113())); // this will cause a lot of pain
            ((CameraAccessor) camera).setDetached(true);
        }
    }

    @Inject(method = "renderLevel", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/Camera;isDetached()Z", shift = At.Shift.AFTER))
    private void resetThirdPerson(class_9779 deltaTracker, boolean bl, class_4184 camera, class_757 gameRenderer, class_765 lightTexture, Matrix4f matrix4f, Matrix4f matrix4f2, CallbackInfo ci) {
        ((CameraAccessor) camera).setDetached(defaultCameraState);
    }


    @Inject(method = "renderEntity", at = @At("TAIL"))
    private void dontRenderEntity_End(class_1297 entity, double cameraX, double cameraY, double cameraZ,
                                      float tickDelta, class_4587 matrices, class_4597 vertexConsumers, CallbackInfo ci) {
        class_4184 camera = class_310.method_1551().field_1773.method_19418();
        if (entity == camera.method_19331()) {
            FirstPersonMode.setFirstPersonPass(false); // Unmark this render cycle
        }
    }
}

==> player-animation-lib-2.0.4%2B1.21.1-sources.jar: dev/kosmx/playerAnim/mixin/firstPerson/LevelRendererMixin.java <==
package dev.kosmx.playerAnim.mixin.firstPerson;

import dev.kosmx.playerAnim.api.firstPerson.FirstPersonMode;
import dev.kosmx.playerAnim.impl.IAnimatedPlayer;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_310;
import net.minecraft.class_4184;
import net.minecraft.class_4587;
import net.minecraft.class_4597;
import net.minecraft.class_757;
import net.minecraft.class_761;
import net.minecraft.class_765;
import net.minecraft.class_9779;
import org.joml.Matrix4f;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_761.class)
public class LevelRendererMixin {

    @Unique
    private boolean defaultCameraState = false;

    // @Redirect(at = @At(target = "Lnet/minecraft/client/Camera;isDetached()Z")) is forbidden

    @Inject(method = "renderLevel", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/Camera;isDetached()Z"))
    private void fakeThirdPersonMode(class_9779 deltaTracker, boolean bl, class_4184 camera, class_757 gameRenderer, class_765 lightTexture, Matrix4f matrix4f, Matrix4f matrix4f2, CallbackInfo ci) {
        // mods may need to redirect that method, I want to avoid compatibility issues as long as possible
        defaultCameraState = camera.method_19333();
        if (camera.method_19331() instanceof IAnimatedPlayer player && player.playerAnimator_getAnimation().getFirstPersonMode() == FirstPersonMode.THIRD_PERSON_MODEL) {
            FirstPersonMode.setFirstPersonPass(!camera.method_19333() && (!(camera.method_19331() instanceof class_1309) || !((class_1309)camera.method_19331()).method_6113())); // this will cause a lot of pain
            ((CameraAccessor)camera).setDetached(true);
        }
    }
    @Inject(method = "renderLevel", at = @At(value = "INVOKE", target = "Lnet/minecraft/client/Camera;isDetached()Z", shift = At.Shift.AFTER))
    private void resetThirdPerson(class_9779 deltaTracker, boolean bl, class_4184 camera, class_757 gameRenderer, class_765 lightTexture, Matrix4f matrix4f, Matrix4f matrix4f2, CallbackInfo ci) {
        ((CameraAccessor)camera).setDetached(defaultCameraState);
    }



    @Inject(method = "renderEntity", at = @At("TAIL"))
    private void dontRenderEntity_End(class_1297 entity, double cameraX, double cameraY, double cameraZ,
                                      float tickDelta, class_4587 matrices, class_4597 vertexConsumers, CallbackInfo ci) {
        class_4184 camera = class_310.method_1551().field_1773.method_19418();
        if (entity == camera.method_19331()) {
            FirstPersonMode.setFirstPersonPass(false); // Unmark this render cycle
        }
    }
}
